import java.util.*;
import java.io.*;

public class MyWorld {
   private PrintStream out;
   
   private ArrayList<PhysicsElement> elements;  // array to hold everything in my world.

   public MyWorld(){
      this(System.out);  // delta_t= 0.1[ms] and refreshPeriod=200 [ms]
   }
   public MyWorld(PrintStream output){
      out = output;
      elements = new ArrayList<PhysicsElement>();     
   }

   public void addElement(PhysicsElement e) {
      elements.add(e);
   }

   public void printStateDescription(){
     String s ="Time ";
     for (PhysicsElement e:elements)
       s+=" \t"+e.getDescription() ;
     out.println(s);
   }

   public void printState(double t){
    //  to be coded by you
     String s = " "+t;
     for (PhysicsElement e:elements)
       s+=" \t"+e.getState();
     out.println(s);    
   }

   public void simulate (double delta_t, double endTime, double samplingTime) {  // simulate passing time
      double t=0;
      printStateDescription();
      printState(t);
      while (t<endTime) {
         for(double nextStop=t+samplingTime; t<nextStop; t+=delta_t) {
           for (PhysicsElement e: elements)   
              if (e instanceof Simulateable) {
                Simulateable sim = (Simulateable) e;
                sim.computeNextState(delta_t); // compute each element next state based on current global state
              }
           for (PhysicsElement e: elements)   // for each element update its state. 
              if (e instanceof Simulateable) {
                Simulateable sim = (Simulateable) e;
                sim.updateState();     // update its state
           }
         }
         printState(t);
      }
   }   
} 
